<?php
namespace Sello\ChannelConnector;

if (!defined('ABSPATH') ) {
    exit;
}

class Category
{
    private $table;
    private $integrations;

    public function __construct($integrations)
    {
        global $wpdb;
        $this->table = $wpdb->prefix."sello_channelconnector_categories";
        $this->integrations = $integrations;
    }

    public function init()
    {
        // $this->refresh();
    }

    private function _check_exists()
    {
        global $wpdb;
        if ($wpdb->get_var("SHOW TABLES LIKE '{$this->table}'") != $this->table) {
            $wpdb->query(
                "CREATE TABLE `{$this->table}` (".
                "`id` varchar(30) NOT NULL,".
                "`market_id` int(11) unsigned NOT NULL,".
                "`parent_id` varchar(30) NOT NULL,".
                "`version` int(11) unsigned NOT NULL,".
                "`name` varchar(100) DEFAULT NULL,".
                "`crumb` varchar(255) DEFAULT NULL,".
                "PRIMARY KEY (`id`, `market_id`), ".
                "KEY (`version`), ".
                "KEY (`market_id`, `parent_id`) ".
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8"
            );
            $this->refresh(false);
        }
    }

    public function getCrumb($marketId, $id)
    {
        global $wpdb;
        $this->_check_exists();
        $cat = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT crumb FROM ".$this->table." WHERE market_id=%d AND id=%d LIMIT 1",
                [$marketId, $id]
            ),
            ARRAY_A
        );

        return $cat && count($cat) > 0 ? html_entity_decode($cat[0]['crumb']) : '';
    }

    public function get($marketId, $offset = 0, $limit = 25, $search = '')
    {
        global $wpdb;
        $this->_check_exists();
        if ($search) {
            $bind = [ $marketId, "%".$wpdb->esc_like($search)."%", $offset, $limit ];
        } else {
            $bind = [ $marketId, $offset, $limit ];
        }

        $categories = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT * FROM ".$this->table." WHERE market_id=%d ".($search ? "AND crumb LIKE %s " : "")." ORDER BY crumb ASC LIMIT %d, %d",
                $bind
            ),
            ARRAY_A
        );

        return $categories;
    }

    public function refresh($check = true)
    {
        $ccAccountId = get_option('cc_account_id');
        $ccCategoriesUpdated = get_option('cc_categories_updated');
        if (!$ccAccountId) {
            return;
        }

        global $wpdb;
        if ($check) {
            $this->_check_exists();
        }

        // If there are no categories - update them now
        $count = $wpdb->get_results("SELECT COUNT(*) as num FROM {$this->table}", ARRAY_A);
        if ($check && $ccCategoriesUpdated > time() - (3600 * 24) && $count[0]['num'] > 0) {
            return;
        }
        // Get categories from CC
        $data = \wp_remote_get(
            "https://wc-api.sello.io/categories/".$ccAccountId,
            [
                "headers" => array("Content-type" => "application/json")
            ]
        );
        $categories = json_decode(\wp_remote_retrieve_body($data), true);
        // Set variables for later use
        $version = time();
        $urls = [];
        $ourMarkets = [];
        // Got no categories
        if (empty($categories) || !is_array($categories)) {
            return;
        }
        // Loop categories
        foreach ($categories as $category) {
            // If we have this integration
            foreach ($this->integrations as $ig) {
                if ($ig['market_id'] == $category['marketId']) {
                    $urls[$category['url']] = true;
                    $ourMarkets[$ig['market_id']] = true;
                    break;
                }
            }
        }

        // Download each list and add it to the database
        foreach ($urls as $url => $bool) {
            $res = \wp_remote_get($url);
            $lines = explode("\n", $res['body']);
            $items = [];
            $header = true;
            foreach ($lines as $line) {
                if ($header) {
                    $header = false;
                    continue;
                }

                list($id, $parentId, $name, $crumb) = str_getcsv($line);
                $items[] = "(".
                    "'".$id."',".
                    "MARKETID,".
                    "'".$parentId."',".
                    $version.",".
                    "'".str_replace("'", "", $name)."',".
                    "'".str_replace("'", "", $crumb)."'".
                ")";
            }

            // Add to database in chunks of 1000 rows
            $chunks = array_chunk($items, 1000);

            foreach ($chunks as $chunk) {
                // What markets needs these?
                foreach ($categories as $category) {
                    if ($category['url'] === $url && isset($ourMarkets[$category['marketId']])) {
                        $values = implode(",", $chunk);
                        $values = str_replace("MARKETID", $category['marketId'], $values);
                        $wpdb->query("REPLACE INTO ".$this->table." (id, market_id, parent_id, version, name, crumb) VALUES ".$values);
                    }
                }
            }
        }

        $wpdb->query("DELETE FROM ".$this->table." WHERE version!='".$version."'");
        update_option('cc_categories_updated', time());
    }
}
