<?php
namespace Sello\ChannelConnector;

if (!defined('ABSPATH') ) {
    exit;
}

class Integration
{
    private $table;

    public function __construct()
    {
        global $wpdb;
        $this->table = $wpdb->prefix."sello_channelconnector_integrations";
    }

    public function getAll()
    {
        global $wpdb;
        $this->_check_exists();
        return $wpdb->get_results("SELECT * FROM ".$this->table." ORDER BY display_name ASC", ARRAY_A);
    }

    private function _check_exists()
    {
        global $wpdb;
        if ($wpdb->get_var("SHOW TABLES LIKE '{$this->table}'") != $this->table) {
            $wpdb->query(
                "CREATE TABLE `{$this->table}` (".
                "`id` int(11) unsigned NOT NULL,".
                "`market_id` int(11) unsigned NOT NULL,".
                "`parent_id` int(11) unsigned NOT NULL,".
                "`version` int(11) unsigned NOT NULL,".
                "`display_name` varchar(100) DEFAULT NULL,".
                "PRIMARY KEY (`id`)".
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8"
            );
            $this->refresh(false);
        }
    }

    public function refresh($check = true)
    {
        $ccAccountId = get_option('cc_account_id');
        $ccIntegrationsUpdated = get_option('cc_integrations_updated');
        if (!$ccAccountId) {
            return;
        }
        if ($check && $ccIntegrationsUpdated > time() - 1300) {
            return;
        }

        global $wpdb;
        if ($check) {
            $this->_check_exists();
        }
        try {
            $data = wp_remote_get(
                "https://wc-api.sello.io/integrations/".$ccAccountId,
                [
                    "headers" => array("Content-type" => "application/json")
                ]
            );
            $integrations = json_decode(wp_remote_retrieve_body($data), true);
            // Got invalid data back, or an WP_Error from wp_remote_get
            if (is_null($integrations) || !is_array($integrations)) {
                return;
            }
            $version = time();
            foreach ($integrations as $integration) {
                $wpdb->replace(
                    $this->table,
                    [
                        "id" => $integration["id"],
                        "market_id" => $integration["market_id"],
                        "parent_id" => $integration["parent_id"],
                        "display_name" => $integration["display_name"],
                        "version" => $version,
                    ],
                    [
                        "%d",
                        "%d",
                        "%d",
                        "%s",
                        "%d"
                    ]
                );
            }

            $wpdb->query("DELETE FROM ".$this->table." WHERE version!='".$version."'");
            update_option('cc_integrations_updated', time());
        } catch (\Exception $e) {
            trigger_error("Failed to load integrations for Sello ChannelConnector, got exception: ".$e->getMessage());
        }
    }
}
