<?php
namespace Sello\ChannelConnector\Settings;

if (!defined('ABSPATH')) {
    exit;
}

class Admin
{
    function initMenu()
    {
        add_menu_page('ChannelConnector', 'ChannelConnector', 'manage_options', 'channelconnector', [$this, 'showSettingsPage']);
        add_submenu_page('channelconnector', 'Settings', 'Settings', 'manage_options', 'channelconnector/settings', [$this, 'showSettingsPage']);
    }

    public function showSettingsPage()
    {
        if (isset($_REQUEST['action']) && $_REQUEST['action'] === 'authorize') {
            return $this->AuthorizeAction();
        }

        $apiKey = get_option('cc_account_id');
        if (!$apiKey && !isset($_POST['apiKey'])) {
            $_REQUEST['action'] = "onboarding";
        }

        $validActions = ["home", "setupAccount", "saveSettings", "onboarding", "getPackslip"];
        if (!isset($_REQUEST['action']) || !in_array($_REQUEST['action'], $validActions)) {
            $action = "home";
        } else {
            $action = $_REQUEST['action'];
        }

        $this->{$action."Action"}();
    }

    public function homeAction()
    {
        $currentShippingMethod = get_option('cc_shipping_method');
        $currentPaymentMethod = get_option('cc_payment_method');
        $currentOrderStatus = get_option('cc_new_order_status');
        $useDefaultPaymentMethod = get_option('cc_default_payment_method', true);

        global $woocommerce;
        $shippingMethods = $woocommerce->shipping->get_shipping_methods();

        $p = new \WC_Payment_Gateways();
        $paymentMethods = $p->payment_gateways();

        $orderStatusList = wc_get_order_statuses();
        include "_settings.php";
    }

    public function saveSettingsAction()
    {
        update_option('cc_payment_method', sanitize_text_field($_POST['paymentMethod']));
        update_option('cc_shipping_method', sanitize_text_field($_POST['shippingMethod']));
        update_option('cc_new_order_status', sanitize_text_field($_POST['orderStatus']));
        if (isset($_POST['useDefaultPaymentMethod'])) {
            update_option('cc_default_payment_method', "true");
        } else {
            update_option('cc_default_payment_method', "false");
        }
        Header("Location: ".menu_page_url("channelconnector/settings", false));
    }

    public function onboardingAction()
    {
        include dirname(__FILE__)."/_AdminOnboarding.php";
    }

    public function getPackslipAction()
    {
        $ccAccountId = get_option('cc_account_id');
        $data = wp_remote_post(
            "https://wc-api.sello.io/orders/packslip",
            [
                "headers" => array("Content-type" => "application/json"),
                "body" => json_encode(
                    [
                        "accountId" => $ccAccountId,
                        "orderIds" => [$_GET['orderId']]
                    ]
                ),
                "method" => "POST"
            ]
        );
        $urls = json_decode(\wp_remote_retrieve_body($data), true);
        // Got invalid data back, or an WP_Error from wp_remote_get
        if (is_null($urls) || !is_array($urls)) {
            echo "No packslip found for order " . $_GET['orderId'];
            exit;
        }
        Header("Location: ".$urls['urls'][0]);
        exit;
    }

    public function AuthorizeAction()
    {
        $key = sanitize_text_field(trim($_GET['apiKey']));
        $s = new \Sello\ChannelConnector\Setup();
        $res = $s->createCCAccount($key);
        Header("Location: ".menu_page_url("channelconnector/settings", false).(!$res ? "&invalid=true" : ""));
    }
}
