<?php
namespace Sello\ChannelConnector\Settings;

if (!defined( 'ABSPATH' ) ) {
    exit;
}

class Product
{
    public $integrations;

    public function __construct($integrations)
    {
        $this->integrations = $integrations;
    }

    public function init()
    {
        add_action('woocommerce_product_data_tabs', [$this, 'custom_product_tabs']);
        add_filter('woocommerce_product_data_panels', [$this, 'channelconnector_options_product_tab_content']);
        add_action('woocommerce_process_product_meta_simple', [$this, 'save_channelconnector_option_fields']);
        add_action('woocommerce_process_product_meta_variable', [$this, 'save_channelconnector_option_fields']);
        add_action('woocommerce_product_bulk_edit_start', [$this, 'custom_field_product_bulk_edit'], 10, 0);
        add_action('woocommerce_product_bulk_edit_save', [$this, 'save_custom_field_product_bulk_edit'], 10, 1);
        add_action('wp_ajax_get_categories', [$this, 'getCategories']);
    }

    function custom_field_product_bulk_edit() 
    {
        ?>
        <script type="text/javascript">
        jQuery(document).ready(function ($) {
            <?php foreach ($this->integrations as $ig) { 
                if ($ig['parent_id'] == 0) {
            ?>
            var catselect<?php echo $ig['id'] ?> = $('#channelconnector_category_<?php echo $ig['id']?>').select2({
                width: '100%',
                ajax: {
                    url: '/wp-admin/admin-ajax.php',
                    dataType: 'json',
                    method: 'POST',
                    data: function (params) {
                        return {
                            action: 'get_categories',
                            market: <?php echo $ig['market_id']?>,
                            search: params.term,
                            page: params.page || 1
                        }
                    }
                }
            });
            var option = new Option("<?=__( '— No change —', 'woocommerce' )?>", "", 1, true, true);
            catselect<?php echo $ig['id']?>.append(option).trigger('change');
            catselect<?php echo $ig['id']?>.trigger({
                type: 'select2:select',
                params: {
                    data: { name: "<?=__( '— No change —', 'woocommerce' )?>", id: "" }
                }
            });            
            <?php }} ?>
        });
        </script>

        <?php
        foreach ($this->integrations as $ig) {
        ?>
            <div style="border: 1px solid #ccc; margin-bottom: 10px; padding: 10px;">
                <strong><?php echo $ig['display_name']?></strong>

                <div style="display: flex">
                    <div style="width: 40%"><?=__('Enable', 'channelconnector')?></div>
                    <div style="width: 60%">
                        <select name="channelconnector_active_<?php echo $ig['id']?>">      
                            <option value=""><?=__( '— No change —', 'woocommerce' )?></option>
                            <option value="yes"><?=__( 'Yes', 'channelconnector' )?></option>
                            <option value="no"><?=__( 'No', 'channelconnector' )?></option>
                        </select>
                    </div>
                </div>
                <?php if ($ig['parent_id'] == 0) : ?>
                <div style="display: flex">
                    <div style="width: 40%"><?=__('Category', 'channelconnector')?></div>
                    <div style="width: 60%">
                        <select name="channelconnector_category_<?php echo $ig['id']?>" id="channelconnector_category_<?php echo $ig['id']?>">      
                        </select>
                    </div>
                </div>   
                <?php endif; ?>             
            </div>
        <?php
        }
    }   
    
    function save_custom_field_product_bulk_edit($product)
    {
        $product_id = method_exists( $product, 'get_id' ) ? $product->get_id() : $product->id;

        foreach ($this->integrations as $ig) {
            if (isset($_REQUEST['channelconnector_active_'.$ig['id']]) && in_array($_REQUEST['channelconnector_active_'.$ig['id']], ['yes', 'no'])) {
                update_post_meta(
                    $product_id,
                    'channelconnector_active_'.$ig['id'],
                    $_REQUEST['channelconnector_active_'.$ig['id']]
                );
            }

            if (isset($_REQUEST['channelconnector_category_'.$ig['id']]) && $_REQUEST['channelconnector_category_'.$ig['id']] != "") {
                update_post_meta(
                    $product_id,
                    'channelconnector_category_'.$ig['id'],
                    sanitize_text_field(
                        $_REQUEST['channelconnector_category_'.$ig['id']]
                    )
                );
            }            
        }
    }    

    public function getCategories()
    {
        $limit = 25;
        $offset = isset($_POST['page']) && is_numeric($_POST['page']) ? ($_POST['page']-1)*$limit : 0;
        $search = isset($_POST['search']) ? sanitize_text_field($_POST['search']) : '';
        $c = new \Sello\ChannelConnector\Category(null);
        $categories = $c->get(intval($_POST['market']), $offset, $limit, $search);

        $data = [
            "results" => [],
            "pagination" => [
                "more" => count($categories) === $limit
            ]
        ];

        foreach ($categories as $c) {
            $data["results"][] = [
                "id" => $c['id'],
                "text" => html_entity_decode($c['crumb'])
            ];
        }

        die(json_encode($data));
    }

    function custom_product_tabs( $tabs)
    {
        $accountStatus = get_option('cc_account_status');
        $classes = ['show_if_simple', 'show_if_variable'];
        if ($accountStatus === "trial_expired") {
            echo '<style type="text/css">.trial_expired { background: red; }</style>';
            $classes[] = "trial_expired";
        }

        $tabs['channelconnector'] = array(
            'label' => __('ChannelConnector', 'woocommerce'),
            'target' => 'channelconnector_options',
            'class' => $classes,
        );
    
        return $tabs;
    }

    function channelconnector_options_product_tab_content()
    {
        global $post;

        $category = new \Sello\ChannelConnector\Category(null);
        $metadata = get_metadata('post', $post->ID);
        $accountStatus = get_option('cc_account_status');
        $markets = [1, 6, 13, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 52, 53, 54, 55, 56, 57, 58, 65, 66, 67, 68, 86, 87, 88, 89, 90, 92];

        foreach ($this->integrations as $n => $ig) {
            if ($ig['parent_id'] > 0) {
                foreach ($this->integrations as $i) {
                    if ($i['id'] == $ig['parent_id']) {
                        $this->integrations[$n]['parent_display_name'] = $i['display_name'];
                    }
                }
            }

            $active[$ig['id']] = isset($metadata['channelconnector_active_'.$ig['id']]) ? $metadata['channelconnector_active_'.$ig['id']][0] === "yes" : false;
        }
        ?>

        <div id="channelconnector_options" class="panel woocommerce_options_panel">
            <script type="text/javascript">
            jQuery(document).ready(function ($) {
                <?php foreach ($this->integrations as $ig) { 
                    if ($ig['parent_id'] == 0) {
                ?>
                var catselect<?php echo $ig['id'] ?> = $('#channelconnector_category_<?php echo $ig['id']?>').select2({
                    width: '100%',
                    ajax: {
                        url: '/wp-admin/admin-ajax.php',
                        dataType: 'json',
                        method: 'POST',
                        data: function (params) {
                            return {
                                action: 'get_categories',
                                market: <?php echo $ig['market_id']?>,
                                search: params.term,
                                page: params.page || 1
                            }
                        }
                    }
                });

                // We need to preselect data
                <?php if ($metadata['channelconnector_category_'.$ig['id']] && $metadata['channelconnector_category_'.$ig['id']][0]) {
                    $id = $metadata['channelconnector_category_'.$ig['id']][0];
                    $crumb = str_replace("\"", "", $category->getCrumb($ig['market_id'], $id));
                ?>
                var option = new Option("<?php echo $crumb ?>", "<?php echo $id ?>", 1, true, true);
                catselect<?php echo $ig['id']?>.append(option).trigger('change');
                catselect<?php echo $ig['id']?>.trigger({
                    type: 'select2:select',
                    params: {
                        data: { name: "<?php echo $crumb ?>", id: "<?php echo $id ?>" }
                    }
                });
                <?php } ?>
                <?php } ?>
                <?php } ?>
            });
            </script>

            <?php if ($accountStatus === "trial_expired") : ?>
            <p style="background:#f66; padding: 15px;">Your trial of Sello has expired and functionality has been disabled. <a href="https://sello.io" target="_blank">Please go to Sello</a> to activate your account</p>
            <?php endif; ?>

            <?php if (count($this->integrations) == 0) : ?>
            <div style="background: #ffcece; padding: 15px">
                <?php echo __("You haven't connected any markets to your Sello account yet. Please <a href='https://sello.io' target='_blank'>log in to Sello</a>, click Settings and connect your markets by clicking the logos in the left menu. Once connected, allow up to 30 minutes for them to be synchronized to your shop.", "channelconnector")?>
            </div>
            <?php endif; ?>

            <div style="display:flex; background: #eee; flex-wrap: wrap;">
                <?php foreach ($this->integrations as $ig) : ?>
                <div style="width: 45%; padding: 10px">
                    <div style="background: white; border: 1px solid #ccc; padding: 15px; padding: 10px;">
                        <label style="float: none; width:100%; margin: 0; line-height: 30px;">
                            <?php if (in_array(intval($ig['market_id']), $markets)) : ?><img src="<?php echo plugin_dir_url(realpath(dirname(__FILE__)."/../"))."/assets/img/market/".$ig["market_id"]."_mini.png"?>" style="float:right; padding-right: 10px"><?php endif; ?>
                            <input type="checkbox" name="channelconnector_active_<?php echo $ig['id']?>" style="margin-left: 10px; margin-right: 5px" <?php if ($active[$ig['id']]) { echo " checked"; } ?>>
                            <strong><?php echo __("Enable on", "channelconnector")?> <?php echo $ig['display_name']; ?></strong>
                        </label>
                        <?php if ($ig['parent_id'] == 0) : ?>
                        <p>
                            <select class="select select short" name="channelconnector_category_<?php echo $ig['id']?>" id="channelconnector_category_<?php echo $ig['id']?>">
                            </select>
                        </p>
                        <?php else: ?>
                            <p style="margin:0; line-height: 45px"><?php echo __("Same category as ")?><?=$ig['parent_display_name']?></p>
                        <?php endif; ?>
                    </div>
                </div>
                <?php endforeach; ?>
            </div>
        </div>
        <?php
    }
    
    function save_channelconnector_option_fields( $post_id )
    {
        foreach ($this->integrations as $ig) {
            update_post_meta(
                $post_id,
                'channelconnector_active_'.$ig['id'],
                isset($_POST['channelconnector_active_'.$ig['id']]) ? 'yes' : 'no'
            );
            update_post_meta(
                $post_id,
                'channelconnector_category_'.$ig['id'],
                sanitize_text_field($_POST['channelconnector_category_'.$ig['id']])
            );
        }
    }    
}