<?php
namespace Sello\ChannelConnector;

if (!defined('ABSPATH')) {
    exit;
}

class Setup
{
    public function init() 
    {
        $ccAccountId = get_option('cc_account_id');
        if (!$ccAccountId) {
            return;
        }

        $this->checkApi($ccAccountId);
        $this->checkWebhooks($ccAccountId);
    }

    public function createCCAccount($apiKey)
    {
        $api = $this->createApiKey();

        $data = wp_remote_post(
            "https://wc-api.sello.io/account",
            [
                "headers" => array("Content-type" => "application/json"),
                "body" => json_encode(
                    [
                        "selloApiKey" => $apiKey,
                        "consumerKey" => $api["key"],
                        "consumerSecret" => $api["secret"],
                        "lang" => get_option('WPLANG', 'en_US'),
                        "url" => \get_site_url(),
                    ]
                ),
                "method" => "POST",
                "timeout" => 10
            ]
        );

        if ($data['response']['code'] != 201) {
            return false;
        }

        // Set the id in db
        $res = json_decode($data['body'], true);
        update_option("cc_account_id", $res['id']);

        $this->checkWebhooks($res['id'], true);

        return $res['id'];
    }

    private function checkApi($ccAccountId)
    {
        global $wpdb;

        $result = $wpdb->get_results(
            "SELECT COUNT(*) as num FROM {$wpdb->prefix}woocommerce_api_keys WHERE description='Sello ChannelConnector' AND permissions='read_write'",
            OBJECT
        );

        if ($result[0]->num == 0) {
            $api = $this->createApiKey();

            $data = wp_remote_post(
                "https://wc-api.sello.io/account",
                [
                    "headers" => array("Content-type" => "application/json"),
                    "body" => json_encode(
                        [
                            "id" => $ccAccountId,
                            "consumerKey" => $api["key"],
                            "consumerSecret" => $api["secret"],
                            "url" => \get_site_url()
                        ]
                    ),
                    "method" => "POST",
                    "timeout" => 10
                ]
            );
        }
    }

    public function createApiKey()
    {
        global $wpdb;

        $user = \wp_get_current_user();
        $consumer_key    = 'ck_' . \wc_rand_hash();
        $consumer_secret = 'cs_' . \wc_rand_hash();

        $wpdb->insert(
            $wpdb->prefix . 'woocommerce_api_keys',
            array(
                'user_id'         => $user->ID,
                'description'     => "Sello ChannelConnector",
                'permissions'     => "read_write",
                'consumer_key'    => \wc_api_hash($consumer_key),
                'consumer_secret' => $consumer_secret,
                'truncated_key'   => substr($consumer_key, -7),
            ),
            array(
                '%d',
                '%s',
                '%s',
                '%s',
                '%s',
                '%s',
            )
        );   
        
        return [
            "key" => $consumer_key,
            "secret" => $consumer_secret
        ];
    }

    private function checkWebhooks($ccAccountId, $force = false)
    {
        global $wpdb;

        $expectedTopics = [
            "product.updated" => "https://wc-api.sello.io/products/enqueue/$ccAccountId",
            "order.updated" => "https://wc-api.sello.io/orders/wcupdate/$ccAccountId"
        ];

        // Make sure webhooks are set up
        foreach ($expectedTopics as $topic => $url) {
            $result = $wpdb->get_results(
                $wpdb->prepare(
                    "SELECT webhook_id FROM {$wpdb->prefix}wc_webhooks WHERE delivery_url=%s AND topic=%s AND status='active'",
                    $url,
                    $topic
                ),
                ARRAY_A
            );

            // Remove deprecated product.updated webhook if we have it
            if ($topic == "product.updated") {
                if (count($result) > 0) {
                    $wpdb->query("DELETE FROM {$wpdb->prefix}wc_webhooks WHERE webhook_id={$result[0]['webhook_id']}");
                }
            } else if (count($result) == 0 || $force) {
                $wpdb->insert(
                    $wpdb->prefix."wc_webhooks",
                    [
                        "status" => "active",
                        "name" => "Sello ChannelConnector",
                        "user_id" => 1,
                        "delivery_url" => $url,
                        "date_created" => date("c"),
                        "date_created_gmt" => date("c"),
                        "secret" => "secret",
                        "topic" => $topic,
                        "api_version" => 2
                    ]
                );
                $wpdb->query("DELETE FROM {$wpdb->prefix}options WHERE option_name='_transient_woocommerce_webhook_ids'");
            }
        }

        // This is a bug fix for installs < 1.0.3
        $opt = \get_option("_transient_woocommerce_webhook_ids");
        if (!$opt || !is_array($opt) || count($opt) == 0) {
            $wpdb->query("DELETE FROM {$wpdb->prefix}options WHERE option_name='_transient_woocommerce_webhook_ids'");
        }
    }
}
